# Generated by roxygen2: do not edit by hand

S3method(IC,binreg)
S3method(IC,phreg)
S3method(IC,phreg.par)
S3method(bootstrap,timemets)
S3method(coef,binreg)
S3method(coef,biprobit)
S3method(coef,cor)
S3method(coef,doubleFG)
S3method(coef,mets.twostage)
S3method(coef,phreg)
S3method(coef,phreg.par)
S3method(coef,randomcif)
S3method(coef,randomcifrv)
S3method(coef,survd)
S3method(coef,timemets)
S3method(coef,twinlm)
S3method(coef,twinlm.strata)
S3method(compare,twinlm)
S3method(concordance,cor)
S3method(gof,phreg)
S3method(iid,twinlm)
S3method(lifetable,formula)
S3method(lifetable,matrix)
S3method(lines,phreg)
S3method(logLik,biprobit)
S3method(logLik,phreg.par)
S3method(logLik,twinlm)
S3method(logLik,twinlm.strata)
S3method(model.frame,bptwin)
S3method(model.frame,phreg.par)
S3method(model.frame,twinlm)
S3method(plot,BiRecurrent)
S3method(plot,bptwin)
S3method(plot,casewise)
S3method(plot,claytonoakes)
S3method(plot,covariance.recurrent)
S3method(plot,cumh)
S3method(plot,eventpois)
S3method(plot,gof.phreg)
S3method(plot,mets.twostage)
S3method(plot,phreg)
S3method(plot,predictphreg)
S3method(plot,resmean_phreg)
S3method(plot,timemets)
S3method(plot,twinlm)
S3method(plot,twinlm.strata)
S3method(predict,binreg)
S3method(predict,biprobit)
S3method(predict,mets.twostage)
S3method(predict,phreg)
S3method(predict,phreg.par)
S3method(print,Print)
S3method(print,binreg)
S3method(print,binregTSR)
S3method(print,biprobit)
S3method(print,casewise)
S3method(print,claytonoakes)
S3method(print,cor)
S3method(print,cumh)
S3method(print,daggregate)
S3method(print,do.twinlm.strata)
S3method(print,doubleFG)
S3method(print,dreg)
S3method(print,dtable)
S3method(print,gof.phreg)
S3method(print,mets.twostage)
S3method(print,pc.twostage)
S3method(print,phreg)
S3method(print,phreg.par)
S3method(print,phreg_rct)
S3method(print,predictphreg)
S3method(print,randomcif)
S3method(print,randomcifrv)
S3method(print,resmean_phreg)
S3method(print,summary.binregTSR)
S3method(print,summary.biprobit)
S3method(print,summary.bptwin)
S3method(print,summary.claytonoakes)
S3method(print,summary.cor)
S3method(print,summary.pc.twostage)
S3method(print,summary.phreg)
S3method(print,summary.survivalG)
S3method(print,summary.twinlm)
S3method(print,summary.twinlm.group)
S3method(print,summary.twostageREC)
S3method(print,survd)
S3method(print,timemets)
S3method(print,twinlm)
S3method(print,twinlm.strata)
S3method(print,twostageREC)
S3method(residuals,phreg)
S3method(score,biprobit)
S3method(score,twinlm)
S3method(score,twinlm.strata)
S3method(simulate,bptwin)
S3method(summary,binreg)
S3method(summary,binregTSR)
S3method(summary,biprobit)
S3method(summary,bptwin)
S3method(summary,casewise)
S3method(summary,claytonoakes)
S3method(summary,cor)
S3method(summary,cumh)
S3method(summary,doubleFG)
S3method(summary,dreg)
S3method(summary,gof.phreg)
S3method(summary,mets.twostage)
S3method(summary,pc.twostage)
S3method(summary,phreg)
S3method(summary,phreg_rct)
S3method(summary,randomcif)
S3method(summary,randomcifrv)
S3method(summary,recurrent)
S3method(summary,resmean_phreg)
S3method(summary,survd)
S3method(summary,survivalG)
S3method(summary,timemets)
S3method(summary,twinlm)
S3method(summary,twinlm.strata)
S3method(summary,twostageREC)
S3method(twostage,aalen)
S3method(twostage,cox.aalen)
S3method(twostage,coxph)
S3method(twostage,phreg)
S3method(vcov,binreg)
S3method(vcov,biprobit)
S3method(vcov,doubleFG)
S3method(vcov,phreg)
S3method(vcov,phreg.par)
S3method(vcov,survd)
S3method(vcov,timemets)
S3method(vcov,twinlm)
export("dby2<-")
export("dby<-")
export("dcut<-")
export("ddrop<-")
export("dfactor<-")
export("dkeep<-")
export("dlag<-")
export("dnames<-")
export("dnumeric<-")
export("drelev<-")
export("drelevel<-")
export("drename<-")
export("drm<-")
export("dsort<-")
export("dspline<-")
export("dtrans<-")
export("dtransform<-")
export(BinAugmentCifstrata)
export(Bootcovariancerecurrence)
export(BootcovariancerecurrenceS)
export(BootmediatorSurv)
export(Bootphreg)
export(CCbinomial.twostage)
export(ClaytonOakes)
export(Dbvn)
export(EVaddGam)
export(Effbinreg)
export(EventSplit)
export(FG_AugmentCifstrata)
export(FGprediid)
export(FastCoxPLstrataR)
export(GLprediid)
export(Grandom.cif)
export(IC)
export(IIDbaseline.cifreg)
export(IIDbaseline.phreg)
export(Interval)
export(LinSpline)
export(aalenMets)
export(aalenfrailty)
export(ace.family.design)
export(addCums)
export(alpha2kendall)
export(alpha2spear)
export(ascertained.pairs)
export(back2timereg)
export(basecumhaz)
export(basehazplot.phreg)
export(bicomprisk)
export(bicompriskData)
export(binomial.twostage)
export(binomial.twostage.time)
export(binreg)
export(binregATE)
export(binregATEbin)
export(binregCasewise)
export(binregG)
export(binregTSR)
export(binregt)
export(biprobit)
export(biprobit.time)
export(biprobit.vector)
export(blocksample)
export(bplot)
export(bplotdFG)
export(bptwin)
export(bptwin.time)
export(casewise)
export(casewise.bin)
export(casewise.test)
export(cause.pchazard.sim)
export(cif)
export(cif.yearslost)
export(cifreg)
export(cluster.index)
export(coarse.clust)
export(coefmat)
export(concordanceCor)
export(concordanceTwinACE)
export(concordanceTwostage)
export(cor.cif)
export(corsim.prostate)
export(corsim.prostate.random)
export(count.history)
export(count.historyVar)
export(countID)
export(covIntH1dM1IntH2dM2)
export(covarianceRecurrent)
export(covarianceRecurrentS)
export(covfr)
export(covfridstrata)
export(covfridstrataCov)
export(cpred)
export(cumContr)
export(cumODDS)
export(cumsum2strata)
export(cumsumidstratasum)
export(cumsumidstratasumCov)
export(cumsumstrata)
export(cumsumstratasum)
export(dInterval)
export(daggr)
export(daggregate)
export(dby)
export(dby2)
export(dbyr)
export(dcor)
export(dcount)
export(dcut)
export(ddrop)
export(deval)
export(deval2)
export(dfactor)
export(dhead)
export(diffstrata)
export(divide.conquer)
export(divide.conquer.timereg)
export(dkeep)
export(dlag)
export(dlev)
export(dlevel)
export(dlevels)
export(dlist)
export(dmean)
export(dmeansd)
export(dmvn)
export(dnames)
export(dnumeric)
export(doubleFGR)
export(dprint)
export(dquantile)
export(dreg)
export(drelev)
export(drelevel)
export(drename)
export(dreshape)
export(drm)
export(drop.specials)
export(drop.strata)
export(dsample)
export(dscalar)
export(dsd)
export(dsort)
export(dsort2)
export(dspline)
export(dstr)
export(dsubset)
export(dsum)
export(dsummary)
export(dtab)
export(dtable)
export(dtail)
export(dtrans)
export(dtransform)
export(dunique)
export(easy.binomial.twostage)
export(estimate)
export(eventpois)
export(extendCums)
export(familycluster.index)
export(familyclusterWithProbands.index)
export(fast.approx)
export(fast.cluster)
export(fast.pattern)
export(fast.reshape)
export(faster.reshape)
export(folds)
export(force.same.cens)
export(glm_IPTW)
export(gof)
export(gofG.phreg)
export(gofM.phreg)
export(gofZ.phreg)
export(grouptable)
export(haplo.surv.discrete)
export(headstrata)
export(ilap)
export(indexstrata)
export(indexstratarightR)
export(interval.logitsurv.discrete)
export(invsubdist)
export(ipw)
export(ipw2)
export(jumptimes)
export(kendall.ClaytonOakes.twin.ace)
export(kendall.normal.twin.ace)
export(km)
export(kumarsim)
export(kumarsimRCT)
export(lifecourse)
export(lifetable)
export(lin.approx)
export(logitATE)
export(logitIPCW)
export(logitIPCWATE)
export(logitSurv)
export(loglikMVN)
export(make.pairwise.design)
export(matdoubleindex)
export(matplot.mets.twostage)
export(mdi)
export(mediatorSurv)
export(medweight)
export(mets.options)
export(mlogit)
export(mystrata)
export(mystrata2index)
export(nonparcuminc)
export(normalATE)
export(npc)
export(object.defined)
export(or.cif)
export(or2prob)
export(p11.binomial.twostage.RV)
export(pairRisk)
export(pbvn)
export(pcif)
export(phreg)
export(phreg.par)
export(phregR)
export(phreg_IPTW)
export(phreg_rct)
export(piecewise.data)
export(piecewise.twostage)
export(plack.cif)
export(plack.cif2)
export(plotConfRegion)
export(plotConfRegionSE)
export(plotSurvd)
export(plotcr)
export(plotstrata)
export(pmvn)
export(pre.cifs)
export(pred.cif.boot)
export(predictCumhaz)
export(predictGLM)
export(predictPairPlack)
export(predictSurvd)
export(predictdFG)
export(predictlogitSurvd)
export(predictmlogit)
export(prob.exceed.recurrent)
export(prob.exceedBiRecurrent)
export(prob.exceedBiRecurrentStrata)
export(prob.exceedRecurrent)
export(prob.exceedRecurrentStrata)
export(procform)
export(procform3)
export(procformdata)
export(random.cif)
export(randomDes)
export(rchaz)
export(rchazC)
export(rcrisk)
export(rcrisks)
export(readPhreg)
export(readmargsurv)
export(recmarg)
export(recreg)
export(recregIPCW)
export(recurrentMarginal)
export(recurrentMarginalAIPCW)
export(recurrentMarginalAIPCWdata)
export(recurrentMarginalIPCW)
export(resmean.phreg)
export(resmeanATE)
export(resmeanIPCW)
export(revcumsum)
export(revcumsum2strata)
export(revcumsum2stratafdN)
export(revcumsumidstratasum)
export(revcumsumidstratasumCov)
export(revcumsumstrata)
export(revcumsumstratasum)
export(rmst.phreg)
export(rmstATE)
export(rmstIPCW)
export(rmvn)
export(robust.basehaz.phreg)
export(robust.phreg)
export(rpch)
export(rr.cif)
export(scalecumhaz)
export(scoreMVN)
export(setup.cif)
export(showfitsim)
export(sim.base)
export(sim.cause.cox)
export(sim.cif)
export(sim.cifs)
export(sim.cifsRestrict)
export(sim.cox)
export(simAalenFrailty)
export(simBinFam)
export(simBinFam2)
export(simBinPlack)
export(simClaytonOakes)
export(simClaytonOakes.family.ace)
export(simClaytonOakes.twin.ace)
export(simClaytonOakesLam)
export(simClaytonOakesWei)
export(simCompete.simple)
export(simCompete.twin.ace)
export(simCox)
export(simFrailty.simple)
export(simMultistate)
export(simRecurrent)
export(simRecurrentII)
export(simRecurrentTS)
export(simSurvFam)
export(simTTP)
export(simbinClaytonOakes.family.ace)
export(simbinClaytonOakes.pairs)
export(simbinClaytonOakes.twin.ace)
export(simlogitSurvd)
export(simnordic)
export(simnordic.random)
export(simrchaz)
export(simsubdist)
export(simul.cifs)
export(slope.process)
export(squareintHdM)
export(strataAugment)
export(strataC)
export(subdist)
export(summaryGLM)
export(summaryTimeobject)
export(sumstrata)
export(surv.boxarea)
export(survival.twostage)
export(survivalG)
export(tailstrata)
export(test.conc)
export(tetrachoric)
export(tie.breaker)
export(twin.clustertrunc)
export(twin.polygen.design)
export(twinlm)
export(twinlm.strata)
export(twinlm.time)
export(twinsim)
export(twostage)
export(twostageMLE)
export(twostageREC)
export(vecAllStrata)
import(Rcpp)
import(mvtnorm)
import(splines)
import(stats)
importFrom(grDevices,dev.interactive)
importFrom(grDevices,dev.list)
importFrom(grDevices,devAskNewPage)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,title)
importFrom(lava,"%++%")
importFrom(lava,"%ni%")
importFrom(lava,"addvar<-")
importFrom(lava,"constrain<-")
importFrom(lava,"covariance<-")
importFrom(lava,"distribution<-")
importFrom(lava,"intercept<-")
importFrom(lava,"kill<-")
importFrom(lava,"latent<-")
importFrom(lava,"parameter<-")
importFrom(lava,"regression<-")
importFrom(lava,Col)
importFrom(lava,Expand)
importFrom(lava,IC)
importFrom(lava,Inverse)
importFrom(lava,Model)
importFrom(lava,blockdiag)
importFrom(lava,bootstrap)
importFrom(lava,cancel)
importFrom(lava,compare)
importFrom(lava,confband)
importFrom(lava,constraints)
importFrom(lava,covariance)
importFrom(lava,coxWeibull.lvm)
importFrom(lava,devcoords)
importFrom(lava,endogenous)
importFrom(lava,estimate)
importFrom(lava,eventTime)
importFrom(lava,expit)
importFrom(lava,getoutcome)
importFrom(lava,gof)
importFrom(lava,iid)
importFrom(lava,information)
importFrom(lava,latent)
importFrom(lava,lava.options)
importFrom(lava,logit)
importFrom(lava,lvm)
importFrom(lava,multigroup)
importFrom(lava,pars)
importFrom(lava,regression)
importFrom(lava,revdiag)
importFrom(lava,score)
importFrom(lava,trim)
importFrom(lava,twostage)
importFrom(survival,Surv)
importFrom(survival,concordance)
importFrom(survival,is.Surv)
importFrom(survival,strata)
importFrom(timereg,Cpred)
importFrom(timereg,Event)
importFrom(timereg,predict.two.stage)
importFrom(timereg,sindex.prodlim)
importFrom(timereg,two.stage)
importFrom(utils,capture.output)
importFrom(utils,getS3method)
importFrom(utils,glob2rx)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(mets, .registration=TRUE)
