message("Bivariate Probit models")
example(biprobit)

message("Polygenic models")
example(bptwin)
example(twinlm)

message("Clayton-Oakes, Two-stage")
example(ClaytonOakes)
example(twostage)

message("Pairwise-Odds-Ratio")
example(binomial.twostage)
example(easy.binomial.twostage)

message("Bivariate competing risks")
example(bicomprisk)


