# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.ApplyBy2 <- function(idata, icluster, F, Env, Argument = "x", Columnwise = 0L, Reduce = 0L, epsilon = 1.0e-16) {
    .Call(`_mets_ApplyBy2`, idata, icluster, F, Env, Argument, Columnwise, Reduce, epsilon)
}

.ApplyBy <- function(idata, icluster, f) {
    .Call(`_mets_ApplyBy`, idata, icluster, f)
}

S0FG <- function(ia, iGc, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcstart) {
    .Call(`_mets_S0_FG_GcR`, ia, iGc, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcstart)
}

Gcjumps <- function(iGc, istatus, istrata2, instrata2, iGcstart, injumps) {
    .Call(`_mets_GcjumpsR`, iGc, istatus, istrata2, instrata2, iGcstart, injumps)
}

S0FGN <- function(ia, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcjumps) {
    .Call(`_mets_S0_FGRN`, ia, itype2, istatus, istrata, instrata, istrata2, instrata2, iGcjumps)
}

.scoreMVN <- function(Y, Mu, dMu, S, dS, itol = 0.0) {
    .Call(`_mets_scoreMVN`, Y, Mu, dMu, S, dS, itol)
}

.loglikMVN <- function(Yl, Yu, Status, Mu, S, Threshold, z, su, itol) {
    .Call(`_mets_loglikMVN`, Yl, Yu, Status, Mu, S, Threshold, z, su, itol)
}

.dmvn <- function(u, mu, rho) {
    .Call(`_mets_dmvn`, u, mu, rho)
}

.rmvn <- function(n, mu, rho) {
    .Call(`_mets_rmvn`, n, mu, rho)
}

.logl_weibull <- function(par, entry, exit, status, X, Z) {
    .Call(`_mets_logl_weibull`, par, entry, exit, status, X, Z)
}

.score_weibull <- function(par, entry, exit, status, X, Z, indiv = TRUE) {
    .Call(`_mets_score_weibull`, par, entry, exit, status, X, Z, indiv)
}

# Register entry points for exported C++ functions
methods::setLoadAction(function(ns) {
    .Call(`_mets_RcppExport_registerCCallable`)
})
