% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drop.strata.R
\name{drop.specials}
\alias{drop.specials}
\title{Remove Special Terms from a Formula}
\usage{
drop.specials(x, components, ...)
}
\arguments{
\item{x}{a formula object.}

\item{components}{character vector of special term names to remove.}

\item{...}{additional arguments passed to \code{lava::Specials}.}
}
\value{
The updated formula with an attribute \code{"variables"} containing
  the extracted variable names from removed specials.
}
\description{
Removes terms such as \code{strata()} or \code{cluster()} from a
formula/terms object.
}
