% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{sim_multistateII}
\alias{sim_multistateII}
\title{Illness-Death Competing Risks with Two Causes of Death}
\usage{
sim_multistateII(
  cumhaz,
  death.cumhaz,
  death.cumhaz2,
  n = NULL,
  rr = NULL,
  rd = NULL,
  rd2 = NULL,
  gamma23 = 0,
  gamma24 = 0,
  early2 = 10000,
  gap.time = FALSE,
  max.recurrent = 100,
  cens = NULL,
  rrc = NULL,
  extend = TRUE,
  ...
)
}
\arguments{
\item{cumhaz}{Cumulative hazard from state 1 to 2.}

\item{death.cumhaz}{Cumulative hazard of death from state 1.}

\item{death.cumhaz2}{Cumulative hazard of death from state 2.}

\item{n}{Number of simulations.}

\item{rr}{Relative risks.}

\item{rd}{Relative risks for death from state 1.}

\item{rd2}{Relative risks for death from state 2.}

\item{gamma23}{Early effect parameters for death causes.}

\item{gamma24}{Early effect parameters for death causes.}

\item{early2}{Time threshold for early effect.}

\item{gap.time}{Gap time indicator.}

\item{max.recurrent}{Maximum recurrent events.}

\item{cens}{Censoring specification.}

\item{rrc}{Censoring relative risks.}

\item{extend}{Extend hazards.}

\item{...}{Additional arguments.}
}
\value{
Data frame with multi-state event history.
}
\description{
Simulates data from an illness-death model with two causes of death from the 
illness state. Covariate effects can be introduced via relative risk terms.
}
\author{
Thomas Scheike
}
\keyword{simulation}
\keyword{survival}
