% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-pc-hazard.R
\name{extendCums}
\alias{extendCums}
\title{Extend Cumulative Hazard Functions to Common Time Range}
\usage{
extendCums(cumA, cumB, extend = NULL)
}
\arguments{
\item{cumA}{a cumulative hazard matrix (two columns: time, cumulative hazard)
or a list of such matrices.}

\item{cumB}{an optional second cumulative hazard matrix. If provided it is
combined with \code{cumA} into a list.}

\item{extend}{numeric vector of hazard rates used for extrapolation, one per
cumulative hazard. If \code{NULL} (default), rates are estimated as
the ratio of the last cumulative hazard value to the last time point.}
}
\value{
A list of cumulative hazard matrices extended to the common maximum time.
}
\description{
Extends a collection of cumulative hazard functions so that they all
cover the same time range. Cumulative hazards that end before the
maximum observed time are extrapolated linearly using either a
user-supplied rate or the average hazard rate estimated from the
existing cumulative hazard.
}
\seealso{
\code{\link{rchaz}}, \code{\link{rcrisk}}, \code{\link{mets-simulation}}
}
