% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{hfactioncpx12}
\alias{hfactioncpx12}
\title{hfaction, subset of block randmized study HF-ACtion from WA package}
\format{
Randomized study 
status : 1-event, 2-death, 0-censoring
treatment : 1/0
}
\source{
WA package, Connor et al. 2009
}
\description{
Data from HF-action trial slightly modified from WA package,
consisting of 741 nonischemic patients with baseline
cardiopulmonary test duration less than or equal to 12 minutes.
}
\examples{
data(hfactioncpx12)
}
\keyword{data}
