% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{ttpd}
\alias{ttpd}
\title{ttpd discrete survival data on interval form}
\source{
Simulated data
}
\description{
ttpd discrete survival data on interval form
}
\keyword{data}
