% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{np}
\alias{np}
\title{np data set}
\source{
Simulated data
}
\description{
np data set
}
\keyword{data}
