% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\name{npc}
\alias{npc}
\alias{plotcr}
\alias{nonparcuminc}
\alias{simnordic}
\alias{corsim.prostate}
\alias{alpha2kendall}
\alias{alpha2spear}
\alias{coefmat}
\alias{piecewise.twostage}
\alias{surv.boxarea}
\alias{faster.reshape}
\alias{piecewise.data}
\alias{simBinPlack}
\alias{simBinFam}
\alias{simBinFam2}
\alias{simSurvFam}
\alias{corsim.prostate.random}
\alias{simnordic.random}
\alias{simCox}
\alias{sim}
\alias{grouptable}
\alias{jumptimes}
\alias{folds}
\alias{ace.family.design}
\alias{ascertained.pairs}
\alias{CCbinomial.twostage}
\alias{coarse.clust}
\alias{concordanceTwinACE}
\alias{concordanceTwostage}
\alias{fast.cluster}
\alias{force.same.cens}
\alias{ilap}
\alias{kendall.ClaytonOakes.twin.ace}
\alias{kendall.normal.twin.ace}
\alias{make.pairwise.design}
\alias{make.pairwise.design.competing}
\alias{matplot.mets.twostage}
\alias{object.defined}
\alias{p11.binomial.twostage.RV}
\alias{predictPairPlack}
\alias{simbinClaytonOakes.family.ace}
\alias{simbinClaytonOakes.pairs}
\alias{simbinClaytonOakes.twin.ace}
\alias{simClaytonOakes.family.ace}
\alias{simClaytonOakes.twin.ace}
\alias{simFrailty.simple}
\alias{simCompete.simple}
\alias{simCompete.twin.ace}
\alias{twin.polygen.design}
\alias{procform}
\alias{procform3}
\alias{procformdata}
\alias{drop.specials}
\title{For internal use}
\description{
For internal use
}
\author{
Klaus K. Holst
}
\keyword{utilities}
