% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{mena}
\alias{mena}
\title{Menarche data set}
\source{
Simulated data
}
\description{
Menarche data set
}
\keyword{data}
