% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastcluster.R
\name{fast.cluster}
\alias{fast.cluster}
\title{Fast Cluster Index Conversion}
\usage{
fast.cluster(x, ...)
}
\arguments{
\item{x}{integer vector of cluster identifiers.}

\item{...}{additional arguments (not used).}
}
\value{
Integer vector of 0-based cluster indices.
}
\description{
Converts a cluster variable to consecutive numeric indices using compiled code.
}
