% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{twinbmi}
\alias{twinbmi}
\title{BMI data set}
\format{
Self-reported BMI-values on 11,411 subjects

tvparnr: twin id
bmi: BMI (m/kg^2)
age: Age
gender: (male/female)
zyg: zygosity, MZ:=mz, DZ(same sex):=dz, DZ(opposite sex):=os
}
\description{
BMI data set
}
\keyword{data}
