% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{migr}
\alias{migr}
\title{Migraine data}
\description{
Migraine data
}
\keyword{data}
