% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor.R
\name{summary.cor}
\alias{summary.cor}
\title{Summary for dependence models for competing risks}
\usage{
\method{summary}{cor}(object, marg.cif = NULL, marg.cif2 = NULL, digits = 3, ...)
}
\arguments{
\item{object}{object from cor_cif rr_cif or or_cif for dependence between competing risks data for two causes.}

\item{marg.cif}{a number that gives the cumulative incidence in one time point for which concordance and 
casewise concordance are computed.}

\item{marg.cif2}{the cumulative incidence for cause 2 for concordance and 
casewise concordance are computed. Default is that it is the same as marg.cif.}

\item{digits}{digits in output.}

\item{...}{Additional arguments.}
}
\value{
prints summary for dependence model. 
\item{casewise}{gives casewise concordance that is, probability of cause 2 (related to cif2) given that cause 1 (related to cif1)
	has occured.}
\item{concordance}{gives concordance that is, probability of cause 2 (related to cif2) and cause 1 (related to cif1).}
\item{cif1}{cumulative incidence for cause1.}
\item{cif2}{cumulative incidence for cause1.}
}
\description{
Computes concordance and casewise concordance for dependence models for competing risks 
models of the type cor_cif, rr_cif or or_cif for the given cumulative incidences and the different dependence
measures in the object.
}
\examples{
## library("timereg")
## data("multcif",package="mets") # simulated data 
## multcif$cause[multcif$cause==0] <- 2
##  
## times=seq(0.1,3,by=0.1) # to speed up computations use only these time-points
## add <- timereg::comp.risk(Event(time,cause)~+1+cluster(id),
##                           data=multcif,n.sim=0,times=times,cause=1)
###
## out1<- cor_cif(add,data=multcif,cause1=1,cause2=1,theta=log(2+1))
## summary(out1)
## 
## pad <- predict(add,X=1,se=0,uniform=0)
## summary(out1,marg.cif=pad)
}
\references{
Cross odds ratio Modelling of dependence for
Multivariate Competing Risks Data, Scheike and Sun (2012), Biostatistics. 

A Semiparametric Random Effects Model for Multivariate Competing Risks Data,
Scheike, Zhang, Sun, Jensen (2010), Biometrika.
}
\author{
Thomas Scheike
}
\keyword{survival}
