% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sim-nordic-twin.R
\name{ilap}
\alias{ilap}
\title{Inverse Laplace Transform Helper}
\usage{
ilap(theta, t)
}
\arguments{
\item{theta}{frailty parameter.}

\item{t}{value at which to evaluate.}
}
\value{
Numeric value of the inverse Laplace transform.
}
\description{
Computes the inverse Laplace transform for gamma frailty simulation.
}
