% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mets-package.R
\docType{data}
\name{base4cumhaz}
\alias{base4cumhaz}
\title{rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen}
\source{
Estimated data
}
\description{
rate of Mechanical (hole/defect) complication for catheter of HPN patients of Copenhagen
}
\keyword{data}
