% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/casewise.R
\name{casewise_bin}
\alias{casewise_bin}
\title{Casewise Concordance from Concordant/Discordant Counts}
\usage{
casewise_bin(nc, nd)
}
\arguments{
\item{nc}{number of concordant pairs.}

\item{nd}{number of discordant pairs.}
}
\value{
A list with \code{p.casewise} (estimated probability) and
  \code{ci.casewise} (confidence interval).
}
\description{
Computes casewise concordance probability and confidence interval from
counts of concordant and discordant pairs using a binomial GLM.
}
