% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/restricted.mean.R
\name{ratioATE}
\alias{ratioATE}
\title{Ratio of Average Treatment Effects}
\usage{
ratioATE(rmtl, rmtl1, h = NULL, null = 1)
}
\arguments{
\item{rmtl}{Object containing the ATE for the total RMTL (from \code{resmeanATE}).}

\item{rmtl1}{Object containing the ATE for the specific cause RMTL (from \code{resmeanATE}).}

\item{h}{Transformation function (e.g., \code{log}) applied to the ratio. Default is identity.}

\item{null}{Value under the null hypothesis for the ratio (default 1).}
}
\value{
A list containing:
  \item{ratioG}{Ratio based on the simple IPCW estimator.}
  \item{ratioDR}{Ratio based on the double robust estimator.}
}
\description{
Computes the ratio of two Average Treatment Effects (ATEs), typically comparing the 
ATE for a specific cause (e.g., RMTL due to cause 1) to the ATE for the total RMTL.
}
\details{
The function transforms the estimates (e.g., using log) to compute the ratio and 
its standard error using the delta method and the joint influence functions.
}
\examples{
data(bmt); bmt$event <- bmt$cause!=0; dfactor(bmt) <- tcell~tcell
out <- resmeanATE(Event(time,event)~tcell+platelet, data=bmt, time=40, outcome="rmtl")
out1 <- resmeanATE(Event(time,cause)~tcell+platelet, data=bmt, cause=1, time=40, outcome="rmtl")
ratioATE(out, out1, h=log)
}
\seealso{
\code{\link{resmeanATE}}
}
\author{
Thomas Scheike
}
