% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/phreg.R
\name{IC.phreg}
\alias{IC.phreg}
\title{Influence Functions for phreg objects}
\usage{
\method{IC}{phreg}(x, type = "robust", all = FALSE, time = NULL, baseline = NULL, ...)
}
\arguments{
\item{x}{Object of class \code{"phreg"}.}

\item{type}{Type of influence function: \code{"robust"} (default) or \code{"martingale"}.}

\item{all}{Logical; if \code{TRUE}, returns both beta and baseline influence functions.}

\item{time}{Time point for baseline influence function (required if baseline is requested).}

\item{baseline}{Arguments for baseline estimation.}

\item{...}{Additional arguments.}
}
\value{
A matrix of influence functions. If \code{all=TRUE}, columns correspond to
regression coefficients and baseline cumulative hazard. Attributes include \code{coef}
and \code{time}.
}
\description{
Computes the influence functions (IID decomposition) for the regression coefficients
and/or the baseline cumulative hazard at a specific time point.
}
\seealso{
\code{\link{phreg}}, \code{\link{iidBaseline}}
}
\author{
Thomas Scheike
}
