% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interval-cumoddsreg.r
\name{cumoddsreg}
\alias{cumoddsreg}
\title{Cumulative Odds Regression for Discrete Time Data}
\usage{
cumoddsreg(formula, data, ...)
}
\arguments{
\item{formula}{Formula with a factor response on the left-hand side (representing 
the event time) and covariates on the right.}

\item{data}{Data frame.}

\item{...}{Arguments passed to \code{interval_logitsurv_discrete}.}
}
\value{
An object of class \code{"cumoddsreg"} with the same structure as 
  \code{interval_logitsurv_discrete}.
}
\description{
A wrapper function for \code{interval_logitsurv_discrete} that simplifies the 
interface for discrete time-to-event data where the event time is observed exactly 
or as a factor level. It converts a factor response into an interval-censored format 
internally.
}
\seealso{
\code{\link{interval_logitsurv_discrete}}
}
\author{
Thomas Scheike
}
